@@ -221,8 +221,14 @@
   (error nil :read-only t))
 (declaim (freeze-type undefined-package))
 
-;; Cold load has its own implementation of all symbol fops,
-;; but we have to execute define-fop now to assign their numbers.
+;;; Cold load has its own implementation of all symbol fops,
+;;; but we have to execute define-fop now to assign their numbers.
+;;;
+;;; Any symbols created by the loader must have their SYMBOL-HASH computed.
+;;; This is a requirement for the CASE macro to work. When code is compiled
+;;; to memory, symbols in the expansion are subject to SXHASH, so all is well.
+;;; When loaded, even uninterned symbols need a hash.
+;;; Interned symbols automatically get a precomputed hash.
 (labels #+sb-xc-host ()
         #-sb-xc-host
         ((read-symbol-name (length+flag fasl-input)
@@ -250,12 +256,6 @@
                               (undefined-package-error package)))
                  (push-fop-table (%intern name length package elt-type t)
                                  fasl-input))))
-         ;; Symbol-hash is usually computed lazily and memoized into a symbol.
-         ;; Laziness slightly improves the speed of allocation.
-         ;; But when loading fasls, the time spent in the loader totally swamps
-         ;; any time savings of not precomputing symbol-hash.
-         ;; INTERN hashes everything anyway, so let's be consistent
-         ;; and precompute the hashes of uninterned symbols too.
          (ensure-hashed (symbol)
            (ensure-symbol-hash symbol)
            symbol))
@@ -616,14 +616,19 @@
                                                          (cadr spec)))
                                           (find-layout ',(cadr spec))))
                           specs))))
-  (frob (#x6c t)
-        (#x6d structure-object)
-        (#x6e condition)
-        (#x6f definition-source-location)
-        (#x70 sb-c::debug-fun)
-        (#x71 sb-c::compiled-debug-fun)
-        (#x72 sb-c::debug-info)
-        (#x73 sb-c::compiled-debug-info)
-        (#x74 sb-c::debug-source)
-        (#x75 defstruct-description)
-        (#x76 defstruct-slot-description)))
+  (frob (#x68 t)
+        (#x69 structure-object)
+        (#x6a condition)
+        (#x6b definition-source-location)
+        (#x6c sb-c::debug-info)
+        (#x6d sb-c::compiled-debug-info)
+        (#x6e sb-c::debug-source)
+        (#x6f defstruct-description)
+        (#x70 defstruct-slot-description)
+        (#x71 sb-c::debug-fun)
+        (#x72 sb-c::compiled-debug-fun)
+        (#x73 sb-c::compiled-debug-fun-optional)
+        (#x74 sb-c::compiled-debug-fun-more)
+        (#x75 sb-c::compiled-debug-fun-external)
+        (#x76 sb-c::compiled-debug-fun-toplevel)
+        (#x77 sb-c::compiled-debug-fun-cleanup)))
