@@ -18,8 +18,7 @@
 
 ;;; This gets called by LOAD to resolve newly positioned objects
 ;;; with things (like code instructions) that have to refer to them.
-;;; Return :ABSOLUTE if an absolute fixup needs to be recorded in %CODE-FIXUPS,
-;;; and return :RELATIVE if a relative fixup needs to be recorded.
+;;; Return KIND  if the fixup needs to be recorded in %CODE-FIXUPS.
 ;;; The code object we're fixing up is pinned whenever this is called.
 (defun fixup-code-object (code offset fixup kind flavor)
   (declare (type index offset) (ignorable flavor))
@@ -29,29 +28,26 @@
                        (signed-sap-ref-32 sap offset))
                    fixup)))
     (ecase kind
-        (:absolute64
+      (:absolute64
          ;; Word at sap + offset contains a value to be replaced by
          ;; adding that value to fixup.
          (setf (sap-ref-64 sap offset) fixup))
-        (:absolute
+      (:absolute
          ;; Word at sap + offset contains a value to be replaced by
          ;; adding that value to fixup.
          (setf (sap-ref-32 sap offset) fixup))
-        (:relative
+      (:relative
          ;; Fixup is the actual address wanted.
          ;; Replace word with value to add to that loc to get there.
          ;; In the #-immobile-code case, there's nothing to assert.
-         ;; Relative fixups pretty much can't happen.
+         ;; Relative fixups don't exist with movable code.
          #+immobile-code
          (unless (immobile-space-obj-p code)
            (error "Can't compute fixup relative to movable object ~S" code))
          (setf (signed-sap-ref-32 sap offset)
-               (etypecase fixup
-                 (integer
-                  ;; JMP/CALL are relative to the next instruction,
-                  ;; so add 4 bytes for the size of the displacement itself.
-                  (- fixup
-                     (the (unsigned-byte 64) (+ (sap-int sap) offset 4)))))))))
+               ;; JMP/CALL are relative to the next instruction,
+               ;; so add 4 bytes for the size of the displacement itself.
+               (- fixup (sap-int (sap+ sap (+ offset 4))))))))
   ;; An absolute fixup is stored in the code header's %FIXUPS slot if it
   ;; references an immobile-space (but not static-space) object.
   ;; Note that:
@@ -75,7 +71,7 @@
        (if (eq kind :relative) :relative))))
   nil) ; non-immobile-space builds never record code fixups
 
-#+(or darwin linux openbsd win32)
+#+(or darwin linux openbsd win32 sunos (and freebsd x86-64))
 (define-alien-routine ("os_context_float_register_addr" context-float-register-addr)
   (* unsigned) (context (* os-context-t)) (index int))
 
@@ -84,11 +80,11 @@
 
 (defun context-float-register (context index format)
   (declare (ignorable context index))
-  #-(or darwin linux openbsd win32)
+  #-(or darwin linux openbsd win32 sunos (and freebsd x86-64))
   (progn
     (warn "stub CONTEXT-FLOAT-REGISTER")
     (coerce 0 format))
-  #+(or darwin linux openbsd win32)
+  #+(or darwin linux openbsd win32 sunos (and freebsd x86-64))
   (let ((sap (alien-sap (context-float-register-addr context index))))
     (ecase format
       (single-float
@@ -156,18 +152,33 @@
   (let* ((pc (context-pc context))
          (trap-number (sap-ref-8 pc 0)))
     (declare (type system-area-pointer pc))
-    (if (= trap-number invalid-arg-count-trap)
-        (values #.(error-number-or-lose 'invalid-arg-count-error)
-                '(#.arg-count-sc))
-        (sb-kernel::decode-internal-error-args (sap+ pc 1) trap-number))))
+    (cond ((= trap-number invalid-arg-count-trap)
+           (values #.(error-number-or-lose 'invalid-arg-count-error)
+                   '(#.arg-count-sc)))
+          #+linux
+          ((= trap-number uninitialized-load-trap)
+           (values #.(error-number-or-lose 'uninitialized-memory-error)
+                   (locally
+                       (declare (optimize (safety 0)))
+                     (let* ((data (sap-ref-8 pc 1)) ; encodes dst register and size
+                            (value (sb-vm:context-register context (ash data -2)))
+                            (nbytes (ash 1 (logand data #b11)))
+                            ;; EMIT-SAP-REF always loads the EA into TEMP-REG-TN
+                            ;; which points to the shadow, not the user memory now.
+                            (ea (logxor (sb-vm:context-register
+                                         context (tn-offset sb-vm::temp-reg-tn))
+                                        msan-mem-to-shadow-xor-const)))
+                       `(:raw ,ea ,nbytes ,value)))))
+          (t
+           (sb-kernel::decode-internal-error-args (sap+ pc 1) trap-number)))))
 
 
 #+immobile-code
 (progn
 (defconstant trampoline-entry-offset n-word-bytes)
-(defun fun-immobilize (fun)
+(defun make-simplifying-trampoline (fun)
   (let ((code (truly-the (values code-component &optional)
-                         (sb-vm::alloc-immobile-trampoline))))
+                         (allocate-code-object :dynamic 3 24)))) ; KLUDGE
     (setf (%code-debug-info code) fun)
     (let ((sap (sap+ (code-instructions code) trampoline-entry-offset))
           (ea (+ (logandc2 (get-lisp-obj-address code) lowtag-mask)
@@ -191,6 +202,8 @@
                      (setf (sap-ref-32 sap 7) (logior (ash disp8 24) #x408B48))
                      11))))
         (setf (sap-ref-32 sap i) #xFD60FF))) ; JMP [RAX-3]
+    ;; Verify that the jump table size reads as  0.
+    (aver (zerop (code-jump-table-words code)))
     ;; It is critical that there be a trailing 'uint16' of 0 in this object
     ;; so that CODE-N-ENTRIES reports 0.  By luck, there is exactly enough
     ;; room in the object to hold two 0 bytes. It would be easy enough to enlarge
@@ -201,24 +214,106 @@
 ;;; Return T if FUN can't be called without loading RAX with its descriptor.
 ;;; This is true of any funcallable instance which is not a GF, and closures.
 (defun fun-requires-simplifying-trampoline-p (fun)
-  (cond ((not (immobile-space-obj-p fun)) t) ; always
-        (t
-         (closurep fun))))
+  (let ((kind (fun-subtype fun)))
+    (or (and (eql kind sb-vm:funcallable-instance-widetag)
+             ;; if the FIN has no raw words then it has no internal trampoline
+             (eql (layout-bitmap (%funcallable-instance-layout fun))
+                  sb-kernel::+layout-all-tagged+))
+        (eql kind sb-vm:closure-widetag))))
 
-(defmacro !set-fin-trampoline (fin)
-  `(let ((sap (int-sap (get-lisp-obj-address ,fin)))
-         (insts-offs (- (ash (1+ funcallable-instance-info-offset) word-shift)
-                        fun-pointer-lowtag)))
-     (setf (sap-ref-word sap insts-offs) #xFFFFFFE9058B48 ; MOV RAX,[RIP-23]
-           (sap-ref-32 sap (+ insts-offs 7)) #x00FD60FF))) ; JMP [RAX-3]
+(defconstant +fsc-layout-bitmap+
+  (logxor (1- (ash 1 funcallable-instance-info-offset))
+          (ash 1 (1- funcallable-instance-trampoline-slot))))
+
+;;; This allocator is in its own function because the immobile allocator
+;;; VOPs are impolite (i.e. bad) and trash all registers.
+;;; Since there are no callee-saved registers, this makes it legit'
+;;; to put in a separate function.
+#+immobile-code
+(defun alloc-immobile-funinstance ()
+  (values (%primitive alloc-immobile-fixedobj fun-pointer-lowtag 6 ; kludge
+                      (logior (ash 5 n-widetag-bits) funcallable-instance-widetag))))
+
+;; TODO: put a trampoline in all fins and allocate them anywhere.
+;; Revision e7cd2bd40f5b9988 caused some FINs to go in dynamic space
+;; which is fine, but those fins need to have a default bitmap of -1 instead
+;; of a special bitmap because we examine the bitmap when deciding whether
+;; the FIN can be installed into an FDEFN without needing an external trampoline.
+;; The easiest way to achieve this intent is to default all bitmaps to -1,
+;; then change it in the layout when writing raw words. A better fix would
+;; try to allocate all FINs in immobile space until it is exhausted, then fallback
+;; to dynamic space. The address of the fin is no longer an issue, since fdefns
+;; can point to the entire address space, but the fixed-size immobile object
+;; allocator doesn't returns 0 - it calls the monitor if it fails.
+
+;; So ideally, all funcallable instances would resemble simple-funs for a
+;; small added cost of 2 words per object. It will be necessary to have the GC
+;; treat ambiguous interior pointers to the unboxed words in the same way as
+;; any code pointer. Placing FINs on pages marked as containing code will allow
+;; the conservative root check to be skipped for obviously non-code objects.
+
+;; Also we will need to write the embedded trampoline either in a word index
+;; that differs based on length of the FIN, or place the boxed slots after
+;; the trampoline. As of now, this can only deal with standard GFs.
+;; The primitive object has 2 descriptor slots (fin-fun and CLOS slot vector)
+;; and 2 non-descriptor slots containing machine instructions, after the
+;; self-pointer (trampoline) slot. Scavenging the self-pointer is unnecessary
+;; though harmless. This intricate and/or obfuscated calculation of #b110
+;; is insensitive to the index of the trampoline slot, probably.
+#+immobile-code
+(defun make-immobile-funinstance (layout slot-vector)
+  (let ((gf (truly-the funcallable-instance (alloc-immobile-funinstance))))
+    ;; Ensure that the layout has a bitmap indicating the indices of
+    ;; non-descriptor slots. Doing this just-in-time is easiest, it turns out.
+    (unless (= (layout-bitmap layout) +fsc-layout-bitmap+)
+      (setf (layout-bitmap layout) +fsc-layout-bitmap+))
+    ;; Set layout prior to writing raw slots
+    (setf (%funcallable-instance-layout gf) layout)
+    ;; just being pedantic - liveness is preserved by the stack reference.
+    (with-pinned-objects (gf)
+      (let* ((addr (logandc2 (get-lisp-obj-address gf) lowtag-mask))
+             (sap (int-sap addr))
+             (insts-offs (ash (1+ funcallable-instance-info-offset) word-shift)))
+        (setf (sap-ref-word sap (ash funcallable-instance-trampoline-slot word-shift))
+              (truly-the word (+ addr insts-offs))
+              (sap-ref-word sap insts-offs) #xFFFFFFE9058B48  ; MOV RAX,[RIP-23]
+              (sap-ref-32 sap (+ insts-offs 7)) #x00FD60FF))) ; JMP [RAX-3]
+    (%set-funcallable-instance-info gf 0 slot-vector)
+    gf))
+
+#+immobile-space
+(defun alloc-immobile-fdefn ()
+  (or #+nil ; Avoid creating new objects in the text segment for now
+      (and (= (alien-funcall (extern-alien "lisp_code_in_elf" (function int))) 1)
+           (allocate-immobile-obj (* fdefn-size n-word-bytes)
+                                  (logior (ash undefined-fdefn-header 16)
+                                          fdefn-widetag) ; word 0
+                                  0 other-pointer-lowtag nil)) ; word 1, lowtag, errorp
+      (values (%primitive alloc-immobile-fixedobj other-pointer-lowtag
+                          fdefn-size
+                          (logior (ash undefined-fdefn-header 16)
+                                  fdefn-widetag))))) ; word 0
+
+(defun fdefn-has-static-callers (fdefn)
+  (declare (type fdefn fdefn))
+  (with-pinned-objects (fdefn)
+    (logbitp 7 (sap-ref-8 (int-sap (get-lisp-obj-address fdefn))
+                          (- 1 other-pointer-lowtag)))))
+
+(defun set-fdefn-has-static-callers (fdefn newval)
+  (declare (type fdefn fdefn) (type bit newval))
+  (if (= newval 0)
+      (%primitive unset-fdefn-has-static-callers fdefn)
+      (%primitive set-fdefn-has-static-callers fdefn))
+  fdefn)
 
 (defun %set-fdefn-fun (fdefn fun)
   (declare (type fdefn fdefn) (type function fun)
            (values function))
-  (unless (eql (sb-vm::fdefn-has-static-callers fdefn) 0)
-    (sb-vm::remove-static-links fdefn))
+  (when (fdefn-has-static-callers fdefn)
+    (remove-static-links fdefn))
   (let ((trampoline (when (fun-requires-simplifying-trampoline-p fun)
-                      (fun-immobilize fun)))) ; a newly made CODE object
+                      (make-simplifying-trampoline fun)))) ; a newly made CODE object
     (with-pinned-objects (fdefn trampoline fun)
       (let* ((jmp-target
               (if trampoline
@@ -229,24 +324,8 @@
                   ;; instance w/ builtin trampoline, or a simple-fun.
                   ;; But the result is shifted by N-FIXNUM-TAG-BITS because
                   ;; CELL-REF yields a descriptor-reg, not an unsigned-reg.
-                  (get-lisp-obj-address (%closure-callee fun))))
-             (tagged-ptr-bias
-              ;; compute the difference between the entry address
-              ;; and the tagged pointer to the called object.
-              (the (unsigned-byte 8)
-                   (- jmp-target (get-lisp-obj-address (or trampoline fun)))))
-             (fdefn-addr (- (get-lisp-obj-address fdefn) ; base of the object
-                            other-pointer-lowtag))
-             (jmp-origin ; 5 = instruction length
-              (+ fdefn-addr (ash fdefn-raw-addr-slot word-shift) 5))
-             (jmp-operand
-              (ldb (byte 32 0) (the (signed-byte 32) (- jmp-target jmp-origin))))
-             (instruction
-              (logior #xE9 ; JMP opcode
-                      (ash jmp-operand 8)
-                      (ash #xA890 40) ; "NOP ; TEST %AL, #xNN"
-                      (ash tagged-ptr-bias 56))))
-        (%primitive sb-vm::set-fdefn-fun fdefn fun instruction))))
+                  (get-lisp-obj-address (%closure-callee fun)))))
+        (%primitive set-fdefn-fun fdefn fun jmp-target))))
   fun)
 
 ) ; end PROGN
@@ -270,11 +349,9 @@
 
 ;;; Compute the PC that FDEFN will jump to when called.
 #+immobile-code
-(defun fdefn-call-target (fdefn)
-  (let ((pc (+ (get-lisp-obj-address fdefn)
-               (- other-pointer-lowtag)
-               (ash fdefn-raw-addr-slot word-shift))))
-    (+ pc 5 (signed-sap-ref-32 (int-sap pc) 1)))) ; 5 = length of JMP
+(defun fdefn-raw-addr (fdefn)
+  (sap-ref-word (int-sap (get-lisp-obj-address fdefn))
+                (- (ash fdefn-raw-addr-slot word-shift) other-pointer-lowtag)))
 
 ;;; Undo the effects of XEP-ALLOCATE-FRAME
 ;;; and point PC to FUNCTION
@@ -302,7 +379,6 @@
 ;;; it is impossible to distinguish fdefns that point to the same called function.
 ;;; FIXME: It would be a nice to remove the uniqueness constraint, either
 ;;; by recording ay ambiguous fdefns, or just recording all replacements.
-;;; Perhaps we could remove the static linker mutex as well?
 (defun call-direct-p (fun code-header-funs)
   #-immobile-code (declare (ignore fun code-header-funs))
   #+immobile-code
@@ -325,10 +401,7 @@
 
 ;;; Remove calls via fdefns from CODE when compiling into memory.
 (defun statically-link-code-obj (code fixups)
-  (declare (ignorable fixups))
-  (unless (and (immobile-space-obj-p code)
-               (fboundp 'sb-vm::remove-static-links))
-    (return-from statically-link-code-obj))
+  (declare (ignorable code fixups))
   #+immobile-code
   (let ((insts (code-instructions code))
         (fdefns)) ; group by fdefn
@@ -348,13 +421,13 @@
             ;; Because we're holding the static linker lock, the elements of
             ;; FUNS can not change while this test is performed.
             (when (call-direct-p callee funs)
-              (let ((entry (sb-vm::fdefn-call-target fdefn)))
+              (let ((entry (sb-vm::fdefn-raw-addr fdefn)))
                 (dolist (offset (cdr fdefn-use))
                   ;; Only a CALL or JMP will get statically linked.
                   ;; A MOV will always load the address of the fdefn.
                   (when (eql (logior (sap-ref-8 insts (1- offset)) 1) #xE9)
                     ;; Set the statically-linked flag
-                    (setf (sb-vm::fdefn-has-static-callers fdefn) 1)
+                    (sb-vm::set-fdefn-has-static-callers fdefn 1)
                     ;; Change the machine instruction
                     (setf (signed-sap-ref-32 insts offset)
                           (- entry (+ (sap-int (sap+ insts offset)) 4)))))))))))))
