@@ -345,6 +345,18 @@
                           fill-pointer displaced-to
                           node))
 
+(defoptimizer (make-array-header* derive-type) ((&rest inits))
+  (let* ((data-position #.(sb-vm::slot-offset
+                           (find 'sb-vm::data (sb-vm:primitive-object-slots
+                                               (find 'array sb-vm:*primitive-objects*
+                                                     :key 'sb-vm:primitive-object-name))
+                                 :key 'sb-vm::slot-name)))
+         (data (nth data-position inits))
+         (type (lvar-type data)))
+    (when (array-type-p type)
+      (make-array-type '* :element-type (array-type-element-type type)
+                          :specialized-element-type (array-type-specialized-element-type type)))))
+
 (defoptimizer (%make-array derive-type)
     ((dims widetag n-bits &key adjustable fill-pointer displaced-to
            &allow-other-keys)
@@ -1408,14 +1420,17 @@
               (let ((index-leaf (ref-leaf index-ref)))
                 (loop for constraint in (ref-constraints array-ref)
                       for y = (constraint-y constraint)
-                      thereis (if (constant-p index-leaf)
-                                  (and (constant-p y)
-                                       (<= (constant-value index-leaf)
-                                           (constant-value y)))
-                                  (eq index-leaf y))))
+                      thereis (and
+                               (eq (constraint-kind constraint) 'array-in-bounds-p)
+                               (if (constant-p index-leaf)
+                                   (and (constant-p y)
+                                        (<= (constant-value index-leaf)
+                                            (constant-value y)))
+                                   (eq index-leaf y)))))
               (loop for constraint in (ref-constraints index-ref)
-                    thereis (eq (constraint-y constraint)
-                                (ref-leaf array-ref)))))
+                    thereis (and (eq (constraint-kind constraint) 'array-in-bounds-p)
+                                 (eq (constraint-y constraint)
+                                     (ref-leaf array-ref))))))
       (give-up-ir1-transform)))
   ;; It's in bounds but it may be of the wrong type
   `(the (and fixnum unsigned-byte) index))
@@ -1834,18 +1849,20 @@
 
 ;;; Pick off some constant cases.
 (defoptimizer (array-header-p derive-type) ((array))
-  (let ((type (lvar-type array)))
-    (cond ((not (types-equal-or-intersect type (specifier-type 'simple-array)))
+  (let ((type (lvar-type array))
+        (array-type (specifier-type 'array)))
+    (cond ((or (not (types-equal-or-intersect type array-type))
+               (csubtypep type (specifier-type '(simple-array * (*)))))
+           (specifier-type 'null))
+          ((and (csubtypep type array-type)
+                (not (types-equal-or-intersect type (specifier-type 'simple-array))))
            (specifier-type '(eql t)))
           ((not (array-type-p type))
            ;; FIXME: use analogue of ARRAY-TYPE-DIMENSIONS-OR-GIVE-UP
            nil)
           (t
            (let ((dims (array-type-dimensions type)))
-             (cond ((csubtypep type (specifier-type '(simple-array * (*))))
-                    ;; no array header
-                    (specifier-type 'null))
-                   ((and (listp dims) (/= (length dims) 1))
+             (cond ((and (listp dims) (/= (length dims) 1))
                     ;; multi-dimensional array, will have a header
                     (specifier-type '(eql t)))
                    ((eql (array-type-complexp type) t)
