@@ -716,6 +716,7 @@
 (defstruct (interval (:constructor %make-interval (low high))
                      (:copier nil))
   low high)
+(declaim (freeze-type interval))
 
 #-sb-xc-host ; (See CROSS-FLOAT-INFINITY-KLUDGE.)
 (progn
@@ -1125,7 +1126,7 @@
                    (t (numeric-type-format arg))))
          (bound-type (or format 'float)))
     (cond ((numeric-type-real-p arg)
-           (case (interval-range-info (numeric-type->interval arg) $0.0)
+           (case (interval-range-info> (numeric-type->interval arg) $0.0)
              (+
               ;; The number is positive, so the phase is 0.
               (make-numeric-type :class 'float
