@@ -208,8 +208,7 @@
        (when (eq kind :error)
          (setf (basic-combination-kind call) :full))
        (setf (basic-combination-info call) :full)
-       (rewrite-full-call call)
-       (flush-full-call-tail-transfer call))))
+       (rewrite-full-call call))))
   (annotate-fun-lvar (basic-combination-fun call))
   (values))
 
@@ -350,8 +349,8 @@
                       (primitive-types))
               (let ((n-values (nth-value 1 (values-types
                                             (lvar-derived-type arg)))))
-                (loop repeat n-values
-                      for (prim-type . lvar-type) = (pop types)
+                (loop for (prim-type . lvar-type) = (pop types)
+                      repeat n-values
                       do
                       (primitive-types (or prim-type
                                            *backend-t-primitive-type*))
@@ -388,8 +387,7 @@
            (setf (basic-combination-info call) :full)
            (annotate-fun-lvar (basic-combination-fun call) nil)
            (dolist (arg (reverse args))
-             (annotate-unknown-values-lvar arg t))
-           (flush-full-call-tail-transfer call))))
+             (annotate-unknown-values-lvar arg t)))))
 
   (values))
 
@@ -944,7 +942,7 @@
          (:known
           (ltn-analyze-known-call node))))
       (cif (ltn-analyze-if node))
-      (creturn (ltn-analyze-return node))
+      (creturn) ;; delay to FLUSH-FULL-CALL-TAIL-TRANSFERS
       ((or bind entry))
       (exit (ltn-analyze-exit node))
       (cset (ltn-analyze-set node))
@@ -981,6 +979,7 @@
       (setf (block-info block) (make-ir2-block block)))
     (do-blocks (block component)
       (ltn-analyze-block block))
+    (flush-full-call-tail-transfers component)
     (do-blocks (block component)
       (let ((2block (block-info block)))
         (let ((popped (ir2-block-popped 2block)))
@@ -988,6 +987,35 @@
             (push block (ir2-component-values-receivers 2comp)))))))
   (values))
 
+;;; Delay this step till the end, when all the calls are processed and
+;;; and USE-STANDARD-RETURNS can use standard returns if all returns
+;;; are tail-calls.
+(defun flush-full-call-tail-transfers (component)
+  (assign-returns component)
+  (do-blocks (block component)
+    (do* ((node (block-start-node block)
+                (ctran-next ctran))
+          (ctran (node-next node) (node-next node)))
+         (nil)
+      (typecase node
+        (basic-combination
+         (case (basic-combination-info node)
+           (:full
+            (flush-full-call-tail-transfer node)))))
+      (when (eq node (block-last block))
+        (return))))
+  (ltn-analyze-returns component))
+
+(defun assign-returns (component)
+  (dolist (fun (component-lambdas component))
+    (assign-return-locations fun)))
+
+(defun ltn-analyze-returns (component)
+  (dolist (fun (component-lambdas component))
+    (let ((return (lambda-return fun)))
+      (when return
+        (ltn-analyze-return return)))))
+
 ;;; This function is used to analyze blocks that must be added to the
 ;;; flow graph after the normal LTN phase runs. Such code is
 ;;; constrained not to use weird unknown values (and probably in lots
